import numpy as np
import pandas as pd
import os
from sklearn.model_selection import train_test_split
from pykeen.triples import TriplesFactory
from pykeen.pipeline import pipeline
import pyiwn
from itertools import combinations

beng_iwn = pyiwn.IndoWordNet(lang = pyiwn.Language.BENGALI)
all_syns = beng_iwn.all_synsets()

edges = pd.read_csv("../data/BengIwnData/edges.csv", header = None, delimiter = '\t', names = ['ent1','ent2', 'rel'])

X = pd.DataFrame()
X['ent1'] = edges['ent1']
X['rel'] = edges['rel']
X['ent2'] = edges['ent2']
print("The dataframe X: \n", X.head())
X = X.astype(str)

print("Value counts for relation:")
print(X['rel'].value_counts())

print("Changing the edge structure from (synset -> synset) to (word -> word)")

# converting synsets list into a hashmap
syn_dict = dict()
for val in all_syns:
  syn_dict[val.synset_id()] = val
  
def generate_triples(tuple_list, rel_id):
  triple_list = []
  for tup in tuple_list:
    triple = (tup[0], rel_id, tup[1])
    triple_list.append(triple)
  return triple_list

def generate_allcombs(list1, list2):
  res = []
  for val1 in list1:
    for val2 in list2:
      tup = (val1, val2)
      res.append(tup)
  return res
  
  
word_edges = set()
visited_syns = set()
X = X.astype(int)
for i, row in X.iterrows():
  syn1_id = row['ent1']
  syn1 = syn_dict[syn1_id]
  word_list1 = syn1.lemma_names()
  rel = row['rel']
  syn2_id = row['ent2']
  syn2 = syn_dict[syn2_id]
  word_list2 = syn2.lemma_names()

  if(syn1_id not in visited_syns):
    # generating tuples for synonymy relation - id = 31
    synonymy_combs1 = combinations(word_list1, 2)
    synonymy_triples1 = generate_triples(synonymy_combs1, '31') # add to set
    for i, val in enumerate(synonymy_triples1):
      word_edges.add(val)
    visited_syns.add(syn1_id)

  if(syn2_id not in visited_syns):
    synonymy_combs2 = combinations(word_list2, 2)
    synonymy_triples2 = generate_triples(synonymy_combs2, '31') # add to set
    for i, val in enumerate(synonymy_triples2):
      word_edges.add(val)
    visited_syns.add(syn2_id)

  # enumerating the same relation for all words in both synset lists
  word_combinations = generate_allcombs(word_list1, word_list2)
  word_triples = generate_triples(word_combinations, str(rel)) # add to set

  for i, val in enumerate(word_triples):
    word_edges.add(val)

print(f"Number of edges with words as nodes = {len(word_edges)}")
word_edgelist = list(word_edges)  
triples = np.asarray(word_edgelist)

print("triples' shape: ", triples.shape)

# entity dictionary
words = beng_iwn.all_words()
ent_to_id = dict()
i = 0
for word in words:
  ent_to_id[word] = i
  i+=1

# relation dictionary
rel_to_id = dict()
for i in range(1,32):
  rel_to_id[str(i)] = i


tf = TriplesFactory.from_labeled_triples(triples)
print(tf)

train, test, val = tf.split([0.8, 0.1, 0.1])

from pykeen.hpo import hpo_pipeline
hpo_pipeline_result = hpo_pipeline(
    training = train,
    testing = test,
    validation = val,
    model = 'HolE',
    n_trials =300,
    model_kwargs=dict(embedding_dim=100),
    training_kwargs = dict(num_epochs =10),
    stopper='early'
)
print(hpo_pipeline_result)
hpo_pipeline_result.save_to_directory("./hole")

